@@ -29,12 +29,12 @@ class DynRoutData():
     
     def __init__(self):
         self.frameRate = 60
-        self.engagedThresh = 10
     
     
-    def loadBehavData(self,filePath,h5pyFile=None):
+    def loadBehavData(self,filePath,h5pyFile=None,engagedThresh=None,lightLoad=False):
 
         self.behavDataPath = filePath
+        self.engagedThresh = engagedThresh
 
         if h5pyFile and isinstance(h5pyFile,h5py.File):
             # allow an already-open h5py File instance to be used,
@@ -54,20 +54,21 @@ class DynRoutData():
             self.taskVersion = d['taskVersion'].asstr()[()] if 'taskVersion' in d else None
             self.startTime = d['startTime'].asstr()[()]
             
-            self.frameIntervals = d['frameIntervals'][:]
-            self.frameTimes = np.concatenate(([0],np.cumsum(self.frameIntervals)))
-            self.lastFrame =d['lastFrame'][()] if 'lastFrame' in d else None
-            if self.lastFrame is not None and self.lastFrame != self.frameIntervals.size:
-                print('\n',self.subjectName,self.startTime,'n frames',self.lastFrame,self.frameIntervals.size,'\n')
+            frameIntervals = d['frameIntervals'][:]
+            frameTimes = np.concatenate(([0],np.cumsum(frameIntervals)))
+            self.frameTimes = None if lightLoad else frameTimes
+            self.lastFrame = d['lastFrame'][()] if 'lastFrame' in d else None
+            if self.lastFrame is not None and self.lastFrame != frameIntervals.size:
+                print('\n',self.subjectName,self.startTime,'n frames',self.lastFrame,frameIntervals.size,'\n')
             
             self.endsWithNonCompletedTrial = d['trialStartFrame'].size > d['trialEndFrame'].size
             self.trialEndFrame = d['trialEndFrame'][:]
-            self.trialEndTimes = self.frameTimes[self.trialEndFrame]
+            self.trialEndTimes = frameTimes[self.trialEndFrame]
             self.nTrials = self.trialEndFrame.size
             self.trialStartFrame = d['trialStartFrame'][:self.nTrials]
-            self.trialStartTimes = self.frameTimes[self.trialStartFrame]
+            self.trialStartTimes = frameTimes[self.trialStartFrame]
             self.stimStartFrame = d['trialStimStartFrame'][:self.nTrials]
-            self.stimStartTimes = self.frameTimes[self.stimStartFrame]
+            self.stimStartTimes = frameTimes[self.stimStartFrame]
             
             self.newBlockAutoRewards = d['newBlockAutoRewards'][()]
             self.newBlockGoTrials = d['newBlockGoTrials'][()]
@@ -80,7 +81,8 @@ class DynRoutData():
             self.incorrectTimeoutFrames = d['incorrectTimeoutFrames'][()]
             
             self.quiescentFrames = d['quiescentFrames'][()]
-            self.quiescentViolationFrames = d['quiescentViolationFrames'][:] if 'quiescentViolationFrames' in d.keys() else d['quiescentMoveFrames'][:]    
+            self.quiescentViolationFrames = d['quiescentViolationFrames'][:] if 'quiescentViolationFrames' in d.keys() else d['quiescentMoveFrames'][:]   
+            self.trialQuiescentViolations = [np.sum((self.quiescentViolationFrames >= start) & (self.quiescentViolationFrames <= end)) for start,end in zip(self.trialStartFrame,self.trialEndFrame)]
             
             self.responseWindow = d['responseWindow'][:]
             self.responseWindowTime = np.array(self.responseWindow)/self.frameRate
@@ -94,7 +96,7 @@ class DynRoutData():
             self.rewardedStim = self.blockStimRewarded[self.trialBlock-1]
             
             self.rewardFrames = d['rewardFrames'][:]
-            self.rewardTimes = self.frameTimes[self.rewardFrames]
+            self.rewardTimes = frameTimes[self.rewardFrames]
             self.rewardSize = d['rewardSize'][:]
             self.trialResponse = d['trialResponse'][:self.nTrials]
             self.trialResponseFrame = d['trialResponseFrame'][:self.nTrials]
@@ -113,23 +115,25 @@ class DynRoutData():
                 self.autoRewarded = self.autoRewardScheduled & np.in1d(self.stimStartFrame+self.autoRewardOnsetFrame,self.rewardFrames)
             self.rewardEarned = self.trialRewarded & (~self.autoRewarded)
             
-            
             self.responseTimes = np.full(self.nTrials,np.nan)
-            self.responseTimes[self.trialResponse] = self.frameTimes[self.trialResponseFrame[self.trialResponse].astype(int)] - self.stimStartTimes[self.trialResponse]
+            self.responseTimes[self.trialResponse] = frameTimes[self.trialResponseFrame[self.trialResponse].astype(int)] - self.stimStartTimes[self.trialResponse]
             
             self.lickFrames = d['lickFrames'][:]
             self.minLickInterval = 0.05
             if len(self.lickFrames) > 0:
-                lickTimesDetected = self.frameTimes[self.lickFrames]
+                lickTimesDetected = frameTimes[self.lickFrames]
                 isLick = np.concatenate(([True], np.diff(lickTimesDetected) > self.minLickInterval))
                 self.lickTimes = lickTimesDetected[isLick]
             else:
                 self.lickTimes = np.array([])
             
             if 'rotaryEncoder' in d and isinstance(d['rotaryEncoder'][()],bytes) and d['rotaryEncoder'].asstr()[()] == 'digital':
-                self.runningSpeed = np.concatenate(([np.nan],np.diff(d['rotaryEncoderCount'][:]) * ((2 * np.pi * d['wheelRadius'][()] * self.frameRate) / d['rotaryEncoderCountsPerRev'][()])))
+                runSpeed = np.concatenate(([np.nan],np.diff(d['rotaryEncoderCount'][:]) * ((2 * np.pi * d['wheelRadius'][()] * self.frameRate) / d['rotaryEncoderCountsPerRev'][()])))
+                self.runningSpeed = None if lightLoad else runSpeed
+                self.quiescentRunSpeed = np.array([np.nanmean(runSpeed[sf-self.quiescentFrames:sf]) for sf in self.stimStartFrame]) 
             else:
                 self.runningSpeed = None
+                self.quiescentRunSpeed = None
             
             self.visContrast = d['visStimContrast'][()]
             self.trialVisContrast = d['trialVisStimContrast'][:self.nTrials]
@@ -203,6 +207,8 @@ class DynRoutData():
                     self.trialOptoLabel = np.full(self.nTrials,'no opto',dtype=object)
                     for lbl,ov,gv in zip(self.optoParams['label'],optoVoltage,galvoVoltage):
                         self.trialOptoLabel[(trialOptoVoltage==ov) & np.all(trialGalvoVoltage==gv,axis=1)] = lbl
+                if 'trialOptoItiOnsetFrame' in d:
+                    self.trialOptoItiOnsetFrame = d['trialOptoItiOnsetFrame'][:self.nTrials]
             
         self.catchTrials = self.trialStim == 'catch'
         self.multimodalTrials = np.array(['+' in stim for stim in self.trialStim])
@@ -217,16 +223,17 @@ class DynRoutData():
         
         self.hitTrials = self.goTrials & self.trialResponse
         self.missTrials = self.goTrials & (~self.trialResponse)
-        self.falseAlarmTrials =self. nogoTrials & self.trialResponse
+        self.falseAlarmTrials = self.nogoTrials & self.trialResponse
         self.correctRejectTrials = self.nogoTrials & (~self.trialResponse)
         self.catchResponseTrials = self.catchTrials & self.trialResponse
         
         self.engagedTrials = np.ones(self.nTrials,dtype=bool)
-        for i in range(self.nTrials):
-            r = self.trialResponse[:i+1][self.goTrials[:i+1]]
-            if r.size > self.engagedThresh:
-                if r[-self.engagedThresh:].sum() < 1:
-                    self.engagedTrials[i] = False
+        if self.engagedThresh is not None:
+            for i in range(self.nTrials):
+                r = self.trialResponse[:i+1][self.goTrials[:i+1]]
+                if r.size > self.engagedThresh:
+                    if r[-self.engagedThresh:].sum() < 1:
+                        self.engagedTrials[i] = False
         
         self.catchResponseRate = []
         self.hitRate = []
@@ -257,10 +264,10 @@ class DynRoutData():
     
 
 def calcDprime(hitRate,falseAlarmRate,goTrials,nogoTrials):
-        hr = adjustResponseRate(hitRate,goTrials)
-        far = adjustResponseRate(falseAlarmRate,nogoTrials)
-        z = [scipy.stats.norm.ppf(r) for r in (hr,far)]
-        return z[0]-z[1]
+    hr = adjustResponseRate(hitRate,goTrials)
+    far = adjustResponseRate(falseAlarmRate,nogoTrials)
+    z = [scipy.stats.norm.ppf(r) for r in (hr,far)]
+    return z[0]-z[1]
 
 
 def adjustResponseRate(r,n):
@@ -293,23 +300,27 @@ def getPerformanceStats(df,sessions):
 
 
 def getFirstExperimentSession(df):
-    experimentSessions = np.where(['multimodal' in task
-                                   or 'contrast'in task
-                                   or 'opto' in task
-                                   or 'nogo' in task
-                                   or 'noAR' in task
-                                   or 'rewardOnly' in task
-                                   or 'no reward' in task
-                                   for task in df['task version']])[0]
+    isExp = np.array(['multimodal' in task
+                      or 'contrast'in task
+                      or 'opto' in task
+                      or 'nogo' in task
+                      or 'noAR' in task
+                      or 'rewardOnly' in task
+                      or 'no reward' in task
+                      or 'catchOnly' in task
+                      or 'oneReward' in task
+                      for task in df['task version']])
+    isMusc = np.array(df['muscimol'])
+    experimentSessions = np.where(isExp | isMusc)[0]
     firstExperimentSession = experimentSessions[0] if len(experimentSessions) > 0 else None
     return firstExperimentSession
 
 
 def getSessionsToPass(mouseId,df,sessions,stage,hitThresh=100,dprimeThresh=1.5):
     sessionsToPass = np.nan
-    for sessionInd in sessions:
-        if sessionInd > sessions[0]:
-            hits,dprimeSame,dprimeOther = getPerformanceStats(df,(sessionInd-1,sessionInd))
+    for i,sessionInd in enumerate(sessions):
+        if i > 0:
+            hits,dprimeSame,dprimeOther = getPerformanceStats(df,(sessions[i-1],sessionInd))
             if ((stage in (1,2) and all(h[0] >= hitThresh for h in hits) and all(d[0] >= dprimeThresh for d in dprimeSame)) or
                 (stage==5 and np.all(np.sum((np.array(dprimeSame) >= dprimeThresh) & (np.array(dprimeOther) >= dprimeThresh),axis=1) > 3))):
                 sessionsToPass = np.where(sessions==sessionInd)[0][0] + 1
@@ -320,13 +331,13 @@ def getSessionsToPass(mouseId,df,sessions,stage,hitThresh=100,dprimeThresh=1.5):
     return sessionsToPass
 
 
-def getSessionData(mouseId,startTime):
+def getSessionData(mouseId,startTime,lightLoad=False):
     if not isinstance(startTime,str):
         startTime = startTime.strftime('%Y%m%d_%H%M%S')
     fileName = 'DynamicRouting1_' + str(mouseId) + '_' + startTime + '.hdf5'
     filePath = os.path.join(baseDir,'Data',str(mouseId),fileName)
     obj = DynRoutData()
-    obj.loadBehavData(filePath)
+    obj.loadBehavData(filePath,lightLoad)
     return obj
 
   
@@ -354,7 +365,7 @@ def updateTrainingSummary(mouseIds=None,replaceData=False):
             if replaceData or df is None or np.sum(df['start time']==startTime)==0:
                 try:
                     obj = DynRoutData()
-                    obj.loadBehavData(f)
+                    obj.loadBehavData(f,engagedThresh=10)
                     exps.append(obj)
                 except Exception as err:
                     print('\nerror loading '+f+'\n')
@@ -374,7 +385,8 @@ def updateTrainingSummary(mouseIds=None,replaceData=False):
                         'pass': 0,
                         'ignore': 0,
                         'hab': 0,
-                        'ephys': 0}  
+                        'ephys': 0,
+                        'muscimol': 0}  
                 if df is None:
                     df = pd.DataFrame(data)
                     sessionInd = 0
@@ -396,23 +408,34 @@ def updateTrainingSummary(mouseIds=None,replaceData=False):
                     handOff = False
                     if 'stage 0' in task:
                         passStage = 1
-                        nextTask = 'stage 1 AMN' if regimen > 4 else 'stage 1'
+                        if regimen==9:
+                            nextTask = 'stage 2 AMN'
+                        else:
+                            nextTask = 'stage 1 AMN' if regimen > 4 else 'stage 1'
                     else:
                         if sessionInd > 0:
                             hits,dprimeSame,dprimeOther = getPerformanceStats(df,(sessionInd-1,sessionInd))
                         if 'stage 1' in task:
-                            if 'stage 1' in prevTask and all(h[0] < lowRespThresh for h in hits):
+                            if regimen<9 and 'stage 1' in prevTask and all(h[0] < lowRespThresh for h in hits):
                                 passStage = -1
                                 nextTask = 'stage 0'
                             elif 'stage 1' in prevTask and all(h[0] >= hitThresh for h in hits) and all(d[0] >= dprimeThresh for d in dprimeSame):
                                 passStage = 1
-                                nextTask = 'stage 2 AMN' if regimen > 4 else 'stage 2'
+                                if regimen==9:
+                                    nextTask = 'stage 5 AMN ori'
+                                else:
+                                    nextTask = 'stage 2 AMN' if regimen > 4 else 'stage 2'
                             else:
                                 nextTask = 'stage 1 AMN' if regimen > 4 else 'stage 1'
                         elif 'stage 2' in task:
-                            if 'stage 2' in prevTask and all(h[0] >= hitThresh for h in hits) and all(d[0] >= dprimeThresh for d in dprimeSame):
+                            if regimen==9 and 'stage 2' in prevTask and all(h[0] < lowRespThresh for h in hits):
+                                passStage = -1
+                                nextTask = 'stage 0 AMN'
+                            elif 'stage 2' in prevTask and all(h[0] >= hitThresh for h in hits) and all(d[0] >= dprimeThresh for d in dprimeSame):
                                 passStage = 1
-                                if regimen>6:
+                                if regimen==9:
+                                    nextTask = 'stage 1 AMN'
+                                elif regimen>6:
                                     nextTask = 'stage 5 ori AMN'
                                 elif regimen in (5,6):
                                     nextTask = 'stage variable ori AMN'
@@ -500,7 +523,7 @@ def updateTrainingSummary(mouseIds=None,replaceData=False):
         df.to_excel(writer,sheet_name=obj.subjectName,index=False)
         sheet = writer.sheets[obj.subjectName]
         for col in ('ABCDEFGHIJK'):
-            if col in ('H','I','J','K'):
+            if col in ('H','I','J','K','L'):
                 w = 10
             elif col in ('B','G'):
                 w = 15
@@ -512,10 +535,10 @@ def updateTrainingSummary(mouseIds=None,replaceData=False):
        
     allMiceDf.to_excel(writer,sheet_name='all mice',index=False)
     sheet = writer.sheets['all mice']
-    for col in ('ABCDEFGHIJKLMN'):
+    for col in ('ABCDEFGHIJKLMNOPQR'):
         if col == 'G':
             w = 20
-        elif col == 'N':
+        elif col == 'R':
             w = 30
         else:
             w = 12
@@ -544,14 +567,14 @@ def updateTrainingSummaryNSB():
         for f in set(behavFiles):
             startTime = re.search('.*_([0-9]{8}_[0-9]{6})',f).group(1)
             startTime = pd.to_datetime(startTime,format='%Y%m%d_%H%M%S')
-            if df is None or np.sum(df['start time']==startTime) < 1:
-                try:
+            try:
+                if df is None or np.sum(df['start time']==startTime) < 1:
                     obj = DynRoutData()
-                    obj.loadBehavData(f)
+                    obj.loadBehavData(f,engagedThresh=10)
                     exps.append(obj)
-                except Exception as err:
-                    print('\nerror loading '+f+'\n')
-                    print(repr(err))
+            except Exception as err:
+                print('\nerror loading '+f+'\n')
+                print(repr(err))
         if len(exps) < 1:
             continue
         exps = sortExps(exps)
@@ -567,7 +590,8 @@ def updateTrainingSummaryNSB():
                         'quiescent violations': obj.quiescentViolationFrames.size,
                         'ignore': 0,
                         'hab': 0,
-                        'ephys': 0}  
+                        'ephys': 0,
+                        'muscimol': 0}  
                 if df is None:
                     df = pd.DataFrame(data)
                     sessionInd = 0
@@ -582,7 +606,7 @@ def updateTrainingSummaryNSB():
         df.to_excel(writer,sheet_name=obj.subjectName,index=False)
         sheet = writer.sheets[obj.subjectName]
         for col in ('ABCDEFGHIJK'):
-            if col in ('I','J','K'):
+            if col in ('I','J','K','L'):
                 w = 10
             elif col in ('B','C','H'):
                 w = 15
