@@ -29,7 +29,9 @@ miceToIgnore = summaryDf['wheel fixed'] & summaryDf['cannula']
 
 hasIndirectRegimen = np.array(summaryDf['stage 3 alt'] | summaryDf['stage 3 distract'] | summaryDf['stage 4'] | summaryDf['stage var'])
 
-sessionData = {lbl: [] for lbl in ('noAR',)} #('training','nogo','noAR','rewardOnly','no reward')}
+trainingPhases = ('initial training','after learning','nogo','noAR','rewardOnly','no reward')
+
+sessionData = {lbl: [] for lbl in trainingPhases}
 
 ind = ~hasIndirectRegimen & summaryDf['stage 5 pass'] & summaryDf['moving grating'] & summaryDf['AM noise'] & ~summaryDf['stage 5 repeats'] & ~miceToIgnore
 mice = np.array(summaryDf[ind]['mouse id'])
@@ -41,28 +43,33 @@ for mid in mice:
     if firstExperimentSession is not None:
         sessions[firstExperimentSession:] = False
     sessions = np.where(sessions)[0]
-    sessionsToPass.append(getSessionsToPass(mid,df,sessions,stage=5))
-    sessionData['training'].append([getSessionData(mid,startTime) for startTime in df.loc[sessions,'start time']])
+    sp = getSessionsToPass(mid,df,sessions,stage=5)
+    sessionsToPass.append(sp)
+    sessionData['initial training'].append([getSessionData(mid,startTime,lightLoad=True) for startTime in df.loc[sessions[:5],'start time']])
+    sessionData['after learning'].append([getSessionData(mid,startTime,lightLoad=True) for startTime in df.loc[sessions[sp:sp+5],'start time']])
 
-mice = {
-        #'nogo': np.array(summaryDf[summaryDf['nogo']]['mouse id']),
+mice = {'nogo': np.array(summaryDf[summaryDf['nogo']]['mouse id']),
         'noAR': np.array(summaryDf[summaryDf['noAR']]['mouse id']),
-        #'rewardOnly': np.array(summaryDf[summaryDf['rewardOnly']]['mouse id']),
-        #'no reward': np.array(summaryDf[summaryDf['no reward']]['mouse id']),
-       }
+        'rewardOnly': np.array(summaryDf[summaryDf['rewardOnly']]['mouse id']),
+        'no reward': np.array(summaryDf[summaryDf['no reward']]['mouse id'])}
 for lbl,mouseIds in mice.items():
-    for mid in mouseIds:
-        df = drSheets[str(mid)] if str(mid) in drSheets else nsbSheets[str(mid)]
-        sessions = np.array([lbl in task for task in df['task version']]) & ~np.array(df['ignore'].astype(bool))
-        sessionData[lbl].append([getSessionData(mid,startTime) for startTime in df.loc[sessions,'start time']])
+    if lbl in trainingPhases:
+        for mid in mouseIds:
+            df = drSheets[str(mid)] if str(mid) in drSheets else nsbSheets[str(mid)]
+            sessions = np.array([lbl in task for task in df['task version']]) & ~np.array(df['ignore'].astype(bool))
+            sessionData[lbl].append([getSessionData(mid,startTime,lightLoad=True) for startTime in df.loc[sessions,'start time']])
         
 
 # construct regressors
-trainingPhases = ('noAR',) # ('initial training','after learning') + tuple(sessionData.keys())[1:]
+trainingPhases = tuple(sessionData.keys())
 nTrialsPrev = 20
-regressors = ('reinforcement','posReinforcement','negReinforcement',
+regressors = ('timeSinceRewardRewTarget','timeSinceRewardNonrewTarget',
+              'blockTimeRewTarget','blockTimeNonrewTarget',
+              'reinforcement','reinforcementNoForgetting',
+              'posReinforcement','posReinforcementNoForgetting',
+              'negReinforcement','negReinforcementNoForgetting',
               'crossModalReinforcement','crossModalPosReinforcement','crossModalNegReinforcement',
-              'perseveration','reward')
+              'perseveration','perseverationNoForgetting','reward','response')
 
 regData = {phase: {} for phase in trainingPhases}
 for phase in regData:
@@ -78,11 +85,7 @@ for phase in regData:
     regData[phase]['X'] = []
     s = -1
     b = -1
-    for m,exps in enumerate(sessionData['noAR']): #(sessionData['training'] if phase in trainingPhases[:2] else sessionData[phase])):
-        if phase == 'initial training':
-            exps = exps[:5]
-        elif phase == 'after learning':
-            exps = exps[sessionsToPass[m]:]
+    for m,exps in enumerate(sessionData[phase]):
         for sn,obj in enumerate(exps):
             print(phase,m,sn)
             s += 1
@@ -99,47 +102,58 @@ for phase in regData:
                     continue
                 trialInd = np.where(trials)[0]
                 nTrials = trials.sum()
+                rewTarget = obj.blockStimRewarded[blockInd]
+                nonRewTarget = 'sound1' if rewTarget=='vis1' else 'vis1'
                 regData[phase]['X'].append({})
                 for r in regressors:
-                    regData[phase]['X'][-1][r] = np.zeros((nTrials,nTrialsPrev))
-                    for n in range(1,nTrialsPrev+1):
-                        for trial,stim in enumerate(obj.trialStim[trials]):
-                            resp = obj.trialResponse[:trialInd[trial]]
-                            rew = obj.trialRewarded[:trialInd[trial]]
-                            trialStim = obj.trialStim[:trialInd[trial]]
-                            sameStim = trialStim==stim
-                            otherModalTarget = 'vis1' if stim[:-1]=='sound' else 'sound1'
-                            otherModal = trialStim==otherModalTarget
-                            if r=='reinforcement' and sameStim[-n]:
-                                regData[phase]['X'][-1][r][trial,n-1] = 1 if rew[-n] else (-1 if resp[-n] else 0)
-                            elif r=='posReinforcement' and sameStim[-n] and rew[-n]:
-                                regData[phase]['X'][-1][r][trial,n-1] = 1
-                            elif r=='negReinforcement' and sameStim[-n] and resp[-n] and not rew[-n]:
-                                regData[phase]['X'][-1][r][trial,n-1] = 1  
-                            elif r=='crossModalReinforcement' and otherModal[-n]:
-                                regData[phase]['X'][-1][r][trial,n-1] = 1 if rew[-n] else (-1 if resp[-n] else 0)
-                            elif r=='crossModalPosReinforcement' and otherModal[-n] and rew[-n]:
-                                regData[phase]['X'][-1][r][trial,n-1] = 1
-                            elif r=='crossModalNegReinforcement' and otherModal[-n] and resp[-n] and not rew[-n]:
-                                regData[phase]['X'][-1][r][trial,n-1] = 1
-                            elif r=='perseveration' and sameStim[-n] and resp[-n]:
-                                regData[phase]['X'][-1][r][trial,n-1] = 1
-                            # if r=='reinforcement' and sameStim.sum() > n:
-                            #     regData[phase]['X'][-1][r][trial,n-1] = 1 if rew[sameStim][-n] else (-1 if resp[sameStim][-n] else 0)
-                            # elif r=='posReinforcement' and sameStim.sum() > n and rew[sameStim][-n]:
-                            #     regData[phase]['X'][-1][r][trial,n-1] = 1
-                            # elif r=='negReinforcement' and sameStim.sum() > n and resp[sameStim][-n] and not rew[sameStim][-n]:
-                            #     regData[phase]['X'][-1][r][trial,n-1] = 1
-                            # elif r=='crossModalReinforcement' and otherModal.sum() > n:
-                            #     regData[phase]['X'][-1][r][trial,n-1] = 1 if rew[otherModal][-n] else (-1 if resp[otherModal][-n] else 0)
-                            # elif r=='crossModalPosReinforcement' and otherModal.sum() > n and rew[otherModal][-n]:
-                            #     regData[phase]['X'][-1][r][trial,n-1] = 1
-                            # elif r=='crossModalNegReinforcement' and otherModal.sum() > n and resp[otherModal][-n] and not rew[otherModal][-n]:
-                            #     regData[phase]['X'][-1][r][trial,n-1] = 1
-                            # elif r=='perseveration' and sameStim.sum() > n and resp[sameStim][-n]:
-                            #     regData[phase]['X'][-1][r][trial,n-1] = 1   
-                            elif r=='reward' and rew[-n]:
-                                regData[phase]['X'][-1][r][trial,n-1] = 1
+                    if r in ('timeSinceRewardRewTarget','timeSinceRewardNonrewTarget'):
+                        regData[phase]['X'][-1][r] = np.zeros((nTrials,1))
+                        for trial in range(nTrials):
+                            if obj.trialStim[trialInd[trial]]==(rewTarget if r=='timeSinceRewardRewTarget' else nonRewTarget):
+                                lastReward = np.where(obj.trialRewarded[:trialInd[trial]])[0]
+                                lastReward = lastReward[-1] if len(lastReward) > 0 else 0
+                                regData[phase]['X'][-1][r][trial] = obj.stimStartTimes[trialInd[trial]] - obj.stimStartTimes[lastReward]
+                    elif r in ('blockTimeRewTarget','blockTimeNonrewTarget'):
+                        bt = obj.stimStartTimes[trials] - obj.stimStartTimes[obj.trialBlock==blockInd+1][0]
+                        bt /= 600
+                        bt[obj.trialStim[trials]!=(rewTarget if r=='blockTimeRewTarget' else nonRewTarget)] = 0
+                        regData[phase]['X'][-1][r] = bt[:,None]
+                    else:
+                        regData[phase]['X'][-1][r] = np.zeros((nTrials,nTrialsPrev))
+                        for n in range(1,nTrialsPrev+1):
+                            for trial,stim in enumerate(obj.trialStim[trials]):
+                                resp = obj.trialResponse[:trialInd[trial]]
+                                rew = obj.trialRewarded[:trialInd[trial]]
+                                trialStim = obj.trialStim[:trialInd[trial]]
+                                sameStim = trialStim==stim
+                                otherModalTarget = 'vis1' if stim[:-1]=='sound' else 'sound1'
+                                otherModal = trialStim==otherModalTarget
+                                if r=='reinforcement' and sameStim[-n]:
+                                    regData[phase]['X'][-1][r][trial,n-1] = 1 if rew[-n] else (-1 if resp[-n] else 0)
+                                elif r=='posReinforcement' and sameStim[-n] and rew[-n]:
+                                    regData[phase]['X'][-1][r][trial,n-1] = 1
+                                elif r=='negReinforcement' and sameStim[-n] and resp[-n] and not rew[-n]:
+                                    regData[phase]['X'][-1][r][trial,n-1] = 1  
+                                elif r=='crossModalReinforcement' and otherModal[-n]:
+                                    regData[phase]['X'][-1][r][trial,n-1] = 1 if rew[-n] else (-1 if resp[-n] else 0)
+                                elif r=='crossModalPosReinforcement' and otherModal[-n] and rew[-n]:
+                                    regData[phase]['X'][-1][r][trial,n-1] = 1
+                                elif r=='crossModalNegReinforcement' and otherModal[-n] and resp[-n] and not rew[-n]:
+                                    regData[phase]['X'][-1][r][trial,n-1] = 1
+                                elif r=='perseveration' and sameStim[-n] and resp[-n]:
+                                    regData[phase]['X'][-1][r][trial,n-1] = 1
+                                elif r=='reinforcementNoForgetting' and sameStim.sum() > n:
+                                    regData[phase]['X'][-1][r][trial,n-1] = 1 if rew[sameStim][-n] else (-1 if resp[sameStim][-n] else 0)
+                                elif r=='posReinforcementNoForgetting' and sameStim.sum() > n and rew[sameStim][-n]:
+                                    regData[phase]['X'][-1][r][trial,n-1] = 1
+                                elif r=='negReinforcementNoForgetting' and sameStim.sum() > n and resp[sameStim][-n] and not rew[sameStim][-n]:
+                                    regData[phase]['X'][-1][r][trial,n-1] = 1
+                                elif r=='perseverationNoForgetting' and sameStim.sum() > n and resp[sameStim][-n]:
+                                    regData[phase]['X'][-1][r][trial,n-1] = 1   
+                                elif r=='reward' and rew[-n]:
+                                    regData[phase]['X'][-1][r][trial,n-1] = 1
+                                elif r=='response' and resp[-n]:
+                                    regData[phase]['X'][-1][r][trial,n-1] = 1
                 regData[phase]['mouseIndex'].append(m)
                 regData[phase]['sessionIndex'].append(s)
                 regData[phase]['blockIndex'].append(b)
@@ -175,10 +189,12 @@ for phase in ('after learning',):#in trainingPhases:
         
 
 # fit model
+trainingPhases = ('after learning',)
 fitType = 'response'
 fitRegressors = ('posReinforcement','negReinforcement','crossModalPosReinforcement','crossModalNegReinforcement','reward','perseveration')
+fitRegressors = fitRegressors
 holdOutRegressor = ('none',) #+ fitRegressors #+ (('reinforcement','crossModalReinforcement'),('reward','action'))
-regressorColors = ([s for s in 'grmbkcy']+['0.5'])[:len(fitRegressors)]
+regressorColors = ([s for s in 'grmbkcy']+['0.5','0.25'])[:len(fitRegressors)]
 
 accuracy = {phase: {h: [] for h in holdOutRegressor} for phase in trainingPhases}
 trainAccuracy = copy.deepcopy(accuracy)
@@ -194,6 +210,7 @@ for phase in trainingPhases:
     si = np.array(regData[phase]['sessionIndex'])
     for h in holdOutRegressor:
         # predict blocks from each session by fitting all other blocks from the same mouse
+        regr = [r for r in fitRegressors if r!=h and r not in h]
         for m in np.unique(mi):
             print(phase,h,m)
             
@@ -212,17 +229,23 @@ for phase in trainingPhases:
                 y.append([])
                 for b in np.where(si==s)[0]:
                     if len(fitRegressors) - (len(h) if isinstance(h,tuple) else 1) > 1:
-                        x[-1].append(np.concatenate([regData[phase]['X'][b][r] for r in fitRegressors if r!=h and r not in h],axis=1))
+                        x[-1].append(np.concatenate([regData[phase]['X'][b][r] for r in regr],axis=1))
                     else:
-                        x[-1].append(regData[phase]['X'][b][[r for r in fitRegressors if r not in h][0]])
+                        x[-1].append(regData[phase]['X'][b][[r for r in regr][0]])
                     y[-1].append(regData[phase][('trialResponse' if fitType=='response' else 'trialResponseTime')][b])
                 ntrials.append([len(b) for b in x[-1]])
                 x[-1] = np.concatenate(x[-1])
                 y[-1] = np.concatenate(y[-1])
+            mx = np.nanmean(np.concatenate(x),axis=0)
+            sx = np.nanstd(np.concatenate(x),axis=0)
             for i in range(len(x)):
                 trainX = np.concatenate(x[:i]+x[i+1:])
+                trainX -= mx
+                trainX /= sx
                 trainY = np.concatenate(y[:i]+y[i+1:])
                 testX = x[i]
+                testX -= mx
+                testX /= sx
                 testY = y[i]
                 if fitType == 'response':
                     model = LogisticRegression(C=1.0,max_iter=1e3)
