@@ -95,6 +95,8 @@ class TaskControl():
                 self.initAccumulatorInterface(params)
             else:
                 self.saveDir = r"\\allen\programs\mindscope\workgroups\dynamicrouting\DynamicRoutingTask\Data"
+                self.localSaveDir = r"C:\Data"
+                self.savePath = None
                 self.computerName = None
                 self.configPath = None
                 self.rewardVol = None
@@ -106,6 +108,7 @@ class TaskControl():
                     self.drawDiodeBox = True
                     self.diodeBoxSize = 120
                     self.diodeBoxPosition = (900,540)
+                    self.diodeBoxFrameInterval = 3
                     self.behavNidaqDevice = 'Dev0'
                     self.rewardLine = (0,7)
                     self.rewardSoundLine = (2,0)
@@ -133,7 +136,7 @@ class TaskControl():
                         self.optoChannels = {'laser_488': (3,4), 'laser_633': (5,6)}
                     elif self.rigName == 'NP3':
                         self.rotaryEncoderSerialPort = 'COM3'
-                        self.solenoidOpenTime = 0.03
+                        self.solenoidOpenTime = 0.04
                         self.networkNidaqDevices = ['zcDAQ9185-213AB43']
                         self.soundMode = 'daq'
                         self.soundNidaqDevice = 'zcDAQ1Mod1'
@@ -181,7 +184,7 @@ class TaskControl():
                     elif self.rigName == 'B6':
                         self.rotaryEncoderSerialPort = 'COM3'
                         self.solenoidOpenTime = 0.035 # 2.37 uL 3/25/2024
-                        self.soundCalibrationFit = (26.184874388495313,-2.397480288683932,59.6253081914033,)
+                        self.soundCalibrationFit = (26.184874388495313,-2.397480288683932,59.6253081914033)
                 elif self.rigName in ('D1','D2','D3','D4','D5','D6'):
                     self.behavNidaqDevice = 'Dev1'
                     self.rewardLine = (0,7)
@@ -203,7 +206,7 @@ class TaskControl():
                         self.rotaryEncoderSerialPort = 'COM4'
                         self.soundCalibrationFit = (29.935916862098154,-3.749553179059451,58.99606396765416)
                     elif self.rigName == 'D6':
-                        self.rotaryEncoderSerialPort = 'COM4'
+                        self.rotaryEncoderSerialPort = 'COM5'
                         self.soundCalibrationFit = (28.089402607768378,-3.047263934748452,62.41874890735028)
                 elif self.rigName in ('E1','E2','E3','E4','E5','E6'):
                     self.behavNidaqDevice = 'Dev1'
@@ -404,7 +407,8 @@ class TaskControl():
 
         # show new frame
         if self.drawDiodeBox:
-            self._diodeBox.fillColor = -self._diodeBox.fillColor
+            if self._sessionFrame % self.diodeBoxFrameInterval == 0:
+                self._diodeBox.fillColor = -self._diodeBox.fillColor
             self._diodeBox.draw()
         self._win.flip()
 
@@ -451,16 +455,26 @@ class TaskControl():
             raise
         finally:
             if self.saveParams:
-                subjName = '' if self.subjectName is None else self.subjectName + '_'
-                if self.saveDir is not None:
-                    saveDir = os.path.join(self.saveDir,self.subjectName)
-                    if not os.path.exists(saveDir):
-                        os.makedirs(saveDir)
-                    self.savePath = os.path.join(saveDir,self.__class__.__name__ + '_' + subjName + self.startTime + '.hdf5')
-                with h5py.File(self.savePath,'w') as fileOut:
-                    saveParameters(fileOut,self.__dict__)
-                    if self.saveFrameIntervals and self._win is not None:
-                        fileOut.create_dataset('frameIntervals',data=self._win.frameIntervals)
+                if self.savePath is None:
+                    saveDirs = []
+                    savePaths = []
+                    fileName = self.__class__.__name__ + '_' + ('' if self.subjectName is None else self.subjectName + '_') + self.startTime + '.hdf5'
+                    for saveBase in (self.localSaveDir,self.saveDir):
+                        saveDirs.append(os.path.join(saveBase,self.subjectName))
+                        savePaths.append(os.path.join(saveDirs[-1],fileName))
+                else:
+                    saveDirs = [None]
+                    savePaths = [self.savePath]
+                for saveDir,savePath in zip(saveDirs,savePaths):
+                    try:
+                        if saveDir is not None and not os.path.exists(saveDir):
+                            os.makedirs(saveDir)
+                        with h5py.File(savePath,'w') as fileOut:
+                            saveParameters(fileOut,self.__dict__)
+                            if self.saveFrameIntervals and self._win is not None:
+                                fileOut.create_dataset('frameIntervals',data=self._win.frameIntervals)
+                    except:
+                        print('could not save ' + savePath)
             self.startTime = None
         
     
@@ -1295,7 +1309,7 @@ if __name__ == "__main__":
         task.stopNidaqDevice()
     elif params['taskVersion'] == 'sound measure':
         nidaqDevName = 'Dev2'
-        #soundVol = [1]
+        #soundVol = [0.5]
         soundVol = [0,0.01,0.02,0.04,0.08,0.16,0.32,0.64,1]
         soundDur = 5
         soundInterval = 5
